#include "aron_conversions.h"

#include <RobotAPI/libraries/aron/core/navigator/data/AllNavigators.h>


namespace armarx::armem
{

    constexpr const char* DATA_WRAPPER_DATA_FIELD            = "__ARON_DATA";
    constexpr const char* DATA_WRAPPER_TIME_STORED_FIELD     = "__WRITER_METADATA__TIME_STORED";
    constexpr const char* DATA_WRAPPER_TIME_CREATED_FIELD    = "__ENTITY_METADATA__TIME_CREATED";
    constexpr const char* DATA_WRAPPER_TIME_SENT_FIELD       = "__ENTITY_METADATA__TIME_SENT";
    constexpr const char* DATA_WRAPPER_TIME_ARRIVED_FIELD    = "__ENTITY_METADATA__TIME_ARRIVED";
    constexpr const char* DATA_WRAPPER_CONFIDENCE_FIELD      = "__ENTITY_METADATA__CONFIDENCE";

}


void armarx::armem::from_aron(const aron::datanavigator::DictNavigatorPtr& dataWrapped, wm::EntityInstance& e)
{
    wm::EntityInstanceMetadata& metadata = e.metadata();

    if (dataWrapped->hasElement(DATA_WRAPPER_DATA_FIELD))
    {
        e.data() = aron::datanavigator::DictNavigator::DynamicCastAndCheck(dataWrapped->getElement(DATA_WRAPPER_DATA_FIELD));
    }

    auto timeCreated = aron::datanavigator::LongNavigator::DynamicCastAndCheck(dataWrapped->getElement(DATA_WRAPPER_TIME_CREATED_FIELD));
    metadata.timeCreated = Time::microSeconds(timeCreated->toAronLongPtr()->value);

    auto timeSent = aron::datanavigator::LongNavigator::DynamicCastAndCheck(dataWrapped->getElement(DATA_WRAPPER_TIME_SENT_FIELD));
    metadata.timeSent = Time::microSeconds(timeSent->toAronLongPtr()->value);

    auto timeArrived = aron::datanavigator::LongNavigator::DynamicCastAndCheck(dataWrapped->getElement(DATA_WRAPPER_TIME_ARRIVED_FIELD));
    metadata.timeArrived = Time::microSeconds(timeArrived->toAronLongPtr()->value);

    auto confidence = aron::datanavigator::DoubleNavigator::DynamicCastAndCheck(dataWrapped->getElement(DATA_WRAPPER_CONFIDENCE_FIELD));
    metadata.confidence = static_cast<float>(confidence->toAronDoublePtr()->value);
}


void armarx::armem::to_aron(aron::datanavigator::DictNavigatorPtr& a, const wm::EntityInstance& e)
{
    if (e.data())
    {
        a->addElement(DATA_WRAPPER_DATA_FIELD, e.data());
    }

    auto timeWrapped = std::make_shared<aron::datanavigator::LongNavigator>();
    timeWrapped->setValue(Time::now().toMicroSeconds());
    a->addElement(DATA_WRAPPER_TIME_STORED_FIELD, timeWrapped);

    const wm::EntityInstanceMetadata& metadata = e.metadata();
    auto timeCreated = std::make_shared<aron::datanavigator::LongNavigator>();
    timeCreated->setValue(metadata.timeCreated.toMicroSeconds());
    a->addElement(DATA_WRAPPER_TIME_CREATED_FIELD, timeCreated);

    auto timeSent = std::make_shared<aron::datanavigator::LongNavigator>();
    timeSent->setValue(metadata.timeSent.toMicroSeconds());
    a->addElement(DATA_WRAPPER_TIME_SENT_FIELD, timeSent);

    auto timeArrived = std::make_shared<aron::datanavigator::LongNavigator>();
    timeArrived->setValue(metadata.timeArrived.toMicroSeconds());
    a->addElement(DATA_WRAPPER_TIME_ARRIVED_FIELD, timeArrived);

    auto confidence = std::make_shared<aron::datanavigator::DoubleNavigator>();
    confidence->setValue(static_cast<double>(metadata.confidence));
    a->addElement(DATA_WRAPPER_CONFIDENCE_FIELD, confidence);
}
