#include "Segment.h"

#include <ArmarXCore/core/application/properties/PluginAll.h>
#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/libraries/aron/core/navigator/type/container/Object.h>
#include <RobotAPI/libraries/armem/server/MemoryToIceAdapter.h>


namespace armarx::armem::server::segment
{

    wm::CoreSegmentBase::CoreSegmentBase(
        armem::server::MemoryToIceAdapter& iceMemory,
        const std::string& defaultCoreSegmentName,
        aron::typenavigator::ObjectNavigatorPtr coreSegmentAronType,
        int defaultMaxHistorySize) :
        Base(iceMemory),
        p({defaultCoreSegmentName, defaultMaxHistorySize}),
    coreSegmentAronType(coreSegmentAronType)
    {
        Logging::setTag("armarx::armem::wm::Segment");
    }


    wm::CoreSegmentBase::~CoreSegmentBase()
    {
    }


    void wm::CoreSegmentBase::defineProperties(
        armarx::PropertyDefinitionsPtr defs,
        const std::string& prefix)
    {
        ARMARX_CHECK_NOT_NULL(defs);

        defs->optional(p.coreSegmentName, prefix + "CoreSegmentName", "Name of the '" + p.coreSegmentName + "' core segment.");
        defs->optional(p.maxHistorySize, prefix + "MaxHistorySize", "Maximal size of '" + p.coreSegmentName + "' history (-1 for infinite).");
    }


    void wm::CoreSegmentBase::onInit()
    {
        ARMARX_CHECK_NOT_NULL(iceMemory.workingMemory);

        segment = &iceMemory.workingMemory->addCoreSegment(p.coreSegmentName, coreSegmentAronType);
        segment->setMaxHistorySize(p.maxHistorySize);
    }


    wm::ProviderSegmentBase::ProviderSegmentBase(
        armem::server::MemoryToIceAdapter& iceMemory,
        const std::string& defaultProviderSegmentName,
        aron::typenavigator::ObjectNavigatorPtr providerSegmentAronType,
        const std::string& defaultCoreSegmentName,
        aron::typenavigator::ObjectNavigatorPtr coreSegmentAronType,
        int defaultMaxHistorySize) :

        Base(iceMemory),
        p({defaultCoreSegmentName, defaultProviderSegmentName, defaultMaxHistorySize}),
    coreSegmentAronType(coreSegmentAronType),
    providerSegmentAronType(providerSegmentAronType)
    {
        Logging::setTag("armarx::armem::wm::Segment");
    }


    wm::ProviderSegmentBase::~ProviderSegmentBase()
    {
    }


    void wm::ProviderSegmentBase::defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix)
    {
        ARMARX_CHECK_NOT_NULL(defs);

        defs->optional(p.coreSegmentName, prefix + "CoreSegmentName", "Name of the '" + p.coreSegmentName + "' core segment.");
        defs->optional(p.providerSegmentName, prefix + "ProviderSegmentName", "Name of the '" + p.providerSegmentName + "' provider segment.");
        defs->optional(p.maxHistorySize, prefix + "MaxHistorySize", "Maximal size of '" + p.providerSegmentName + "' history (-1 for infinite).");
    }


    void wm::ProviderSegmentBase::onInit()
    {
        ARMARX_CHECK_NOT_NULL(iceMemory.workingMemory);

        if (!iceMemory.workingMemory->hasCoreSegment(p.coreSegmentName))
        {
            coreSegment = &iceMemory.workingMemory->addCoreSegment(p.coreSegmentName, coreSegmentAronType);
            coreSegment->setMaxHistorySize(p.maxHistorySize);
        }
        else
        {
            coreSegment = &iceMemory.workingMemory->getCoreSegment(p.coreSegmentName);
        }

        segment = &coreSegment->addProviderSegment(p.providerSegmentName, providerSegmentAronType);
        segment->setMaxHistorySize(p.maxHistorySize);
    }

}
