#pragma once

#include <map>
#include <vector>
#include <filesystem>

#include <Eigen/Geometry>

#include <IceUtil/Time.h>

#include "RobotAPI/libraries/ArmarXObjects/ObjectID.h"
#include <ArmarXCore/core/PackagePath.h>


namespace armarx::armem::robot
{
    struct RobotDescription
    {
        // IceUtil::Time timestamp;

        std::string name;
        PackagePath xml{"", std::filesystem::path("")};
    };

    struct RobotState
    {
        using JointMap = std::map<std::string, float>;
        using Pose = Eigen::Affine3f;

        IceUtil::Time timestamp;

        Pose globalPose;
        JointMap jointMap;
    };

    struct Robot
    {
        RobotDescription description;
        std::string instance;

        RobotState config;

        IceUtil::Time timestamp;
    };

    using Robots = std::vector<Robot>;
    using RobotDescriptions = std::vector<RobotDescription>;
    using RobotStates = std::vector<RobotState>;

    std::ostream& operator<<(std::ostream &os, const RobotDescription &rhs);

}  // namespace armarx::armem::robot
