/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/Config.h>

#include <RobotAPI/libraries/aron/core/io/dataIO/visitor/Visitor.h>
#include <RobotAPI/libraries/aron/core/io/dataIO/writer/nlohmannJSON/NlohmannJSONWriter.h>

#include <RobotAPI/libraries/aron/core/io/typeIO/visitor/Visitor.h>
#include <RobotAPI/libraries/aron/core/io/typeIO/writer/nlohmannJSON/NlohmannJSONWriter.h>

namespace armarx::aron
{
    class Debug
    {
    public:
        Debug() = delete;

        static std::string AronDataPtrToString(const data::AronDataPtr& data)
        {
            if (!data)
            {
                return "";
            }

            dataIO::writer::NlohmannJSONWriter w;
            dataIO::Visitor::VisitAndSetup(w, data);
            return w.getResult().dump(2);
        }

        static std::string NavigatorPtrToString(const datanavigator::NavigatorPtr& data)
        {
            if (!data)
            {
                return "";
            }

            dataIO::writer::NlohmannJSONWriter w;
            dataIO::Visitor::VisitAndSetup(w, data);
            return w.getResult().dump(2);
        }

        static std::string AronTypePtrToString(const type::AronTypePtr& data)
        {
            typeIO::writer::NlohmannJSONWriter w;
            typeIO::Visitor::VisitAndSetup(w, data);
            return w.getResult().dump(2);
        }

        static std::string NavigatorPtrToString(const typenavigator::NavigatorPtr& data)
        {
            typeIO::writer::NlohmannJSONWriter w;
            typeIO::Visitor::VisitAndSetup(w, data);
            return w.getResult().dump(2);
        }
    };
}
