/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Position.h"

#include <SimoxUtility/meta/type_name.h>


namespace armarx::aron::cppserializer::serializer
{
    // constructors
    PositionSerializer::PositionSerializer(const typenavigator::PositionNavigatorPtr& n) :
        detail::NDArraySerializerBase<typenavigator::PositionNavigator, PositionSerializer>(
            "Eigen::Matrix<" + n->ACCEPTED_TYPE + ", " + simox::alg::to_string(n->ACCEPTED_DIMENSION, ", ") + ">",
            simox::meta::get_type_name<data::AronNDArray>(),
            simox::meta::get_type_name<type::AronPosition>(),
            n)
    {
        ARMARX_CHECK_NOT_NULL(typenavigator);
    }

    CppBlockPtr PositionSerializer::getResetHardBlock(const std::string& accessor) const
    {
        CppBlockPtr block = std::make_shared<CppBlock>();
        block->addLine(accessor + ".setZero();");
        return this->ResolveMaybeResetHardBlock(accessor, block, this->typenavigator);
    }

    CppBlockPtr PositionSerializer::getResetSoftBlock(const std::string& accessor) const
    {
        return getResetHardBlock(accessor);
    }

    CppBlockPtr PositionSerializer::getWriteTypeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = std::make_shared<CppBlock>();
        b->addLine("w.writePosition({" + MAYBE_TO_STRING(typenavigator->getMaybe()) + "}); // of " + accessor);
        return b;
    }

    CppBlockPtr PositionSerializer::getWriteBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = std::make_shared<CppBlock>();
        block_if_data->addLine("w.writeNDArray({" + simox::alg::to_string(typenavigator->ACCEPTED_DIMENSION, ", ") + ", 4}, \"" + typenavigator->ACCEPTED_TYPE + "\", reinterpret_cast<const unsigned char*>(" + accessor + nextEl() + "data())); // of " + accessor);

        return ResolveMaybeWriteBlock(accessor, block_if_data, typenavigator);
    }

    CppBlockPtr PositionSerializer::getReadBlock(const std::string& accessor) const
    {
        CppBlockPtr block_if_data = std::make_shared<CppBlock>();
        block_if_data->addLine("r.readEndNDArray(reinterpret_cast<unsigned char*>(" + accessor + nextEl() + "data())); // of " + accessor);
        return ResolveMaybeReadBlock(accessor, "r.readStartNDArray()", block_if_data, typenavigator);
    }

    CppBlockPtr PositionSerializer::getEqualsBlock(const std::string& accessor, const std::string& otherInstanceAccessor) const
    {
        CppBlockPtr block_if_data = std::make_shared<CppBlock>();
        block_if_data->addLine("if (not (" + accessor + nextEl() + "isApprox(" + otherInstanceAccessor + ")))");
        block_if_data->addLineAsBlock("return false;");
        return ResolveMaybeEqualsBlock(accessor, otherInstanceAccessor, block_if_data, typenavigator);
    }
}

