/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "Double.h"

#include <SimoxUtility/meta/type_name.h>


namespace armarx::aron::cppserializer::serializer
{
    /* constructors */
    DoubleSerializer::DoubleSerializer(const typenavigator::DoubleNavigatorPtr& e) :
        detail::PrimitiveSerializerBase<typenavigator::DoubleNavigator, DoubleSerializer>(
            "double",
            simox::meta::get_type_name<data::AronDouble>(),
            simox::meta::get_type_name<type::AronDouble>(),
            e)
    {
        ARMARX_CHECK_NOT_NULL(typenavigator);
    }

    /* virtual implementations */
    CppBlockPtr DoubleSerializer::getWriteTypeBlock(const std::string& accessor) const
    {
        CppBlockPtr b = CppBlockPtr(new CppBlock());
        b->addLine("w.writeDouble({" + MAYBE_TO_STRING(typenavigator->getMaybe()) + "}); // of " + accessor);
        return b;
    }
}
