/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <stack>

// BaseClass
#include <RobotAPI/libraries/aron/core/io/dataIO/Reader.h>

// ArmarX
#include <RobotAPI/libraries/aron/core/navigator/data/Navigator.h>
#include <RobotAPI/libraries/aron/core/io/dataIO/reader/navigator/NavigatorReaderToken.h>


namespace armarx::aron::dataIO::reader
{
    class NavigatorReader;
    typedef std::shared_ptr<NavigatorReader> NavigatorReaderPtr;

    class NavigatorReader :
        virtual public ReaderInterface
    {
    public:
        using PointerType = NavigatorReaderPtr;

        // constructors
        NavigatorReader() = delete;
        NavigatorReader(const datanavigator::DictNavigatorPtr& n);
        NavigatorReader(const data::AronDictPtr& n);

        virtual ReadStartDictReturnType readStartDict() override;
        virtual bool readEndDict() override;

        virtual ReadStartListReturnType readStartList() override;
        virtual bool readEndList() override;

        virtual ReadStartNDArrayReturnType readStartNDArray() override;
        virtual void readEndNDArray(unsigned char*) override;

        virtual ReadPrimitiveReturnType readStartPrimitive() override;
        virtual void readEndPrimitive(int&) override;
        virtual void readEndPrimitive(long&) override;
        virtual void readEndPrimitive(float&) override;
        virtual void readEndPrimitive(double&) override;
        virtual void readEndPrimitive(std::string&) override;
        virtual void readEndPrimitive(bool&) override;

        virtual std::string readKey() override;
        virtual void loadMember(const std::string&) override;

        virtual void readNull() override;

        virtual data::Descriptor getTypeOfNext() const override;

    private:
        bool readFirstStartDict = false;
        datanavigator::NavigatorPtr input;
        std::stack<NavigatorReaderTokenPtr> stack;
    };
}
