/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "PCLPointCloud.h"

namespace armarx::aron::typenavigator
{
    // constructors
    PCLPointCloudNavigator::PCLPointCloudNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::ePCLPointCloud, path)
    {
    }

    PCLPointCloudNavigator::PCLPointCloudNavigator(const type::AronPCLPointCloudPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::ePCLPointCloud, path),
        detail::NavigatorBase<type::AronPCLPointCloud, PCLPointCloudNavigator>(o)
    {
        checkTypename(this->aron->typeName);
    }

    // operators
    bool PCLPointCloudNavigator::operator==(const PCLPointCloudNavigator& other) const
    {
        if (getMaybe() != other.getMaybe())
        {
            return false;
        }
        return getTypename() == other.getTypename();
    }

    std::string PCLPointCloudNavigator::checkTypename(const std::string& s) const
    {
        for (const auto& [key, list] : ACCEPTED_TYPES)
        {
            if (s == key)
            {
                return key;
            }

            for (const auto& el : list)
            {
                if (s == el)
                {
                    return key;
                }
            }
        }
        throw error::StringNotValidException("PCLPointCloudNavigator", "checkTypename", "The typename is wrong.", s, getPath());
    }

    type::AronPCLPointCloudPtr PCLPointCloudNavigator::toAronPCLPointCloudPtr() const
    {
        return this->aron;
    }

    std::string PCLPointCloudNavigator::getTypename() const
    {
        return this->aron->typeName;
    }

    void PCLPointCloudNavigator::setTypename(const std::string& u)
    {
        this->aron->typeName = checkTypename(u);
    }

    // virtual implementations
    std::string PCLPointCloudNavigator::getName() const
    {
        return "AronPCLPointCloudType<" + this->aron->typeName + ">";
    }
}

