/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::ArmarXObjects::aron
 * @author     Simon Ottenhaus ( simon dot ottenhaus at kit dot edu )
 * @date       2019
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#define BOOST_TEST_MODULE RobotAPI::ArmarXLibraries::aron

#define ARMARX_BOOST_TEST

// STD/STL
#include <iostream>
#include <cstdlib>
#include <ctime>
#include <numeric>

// Test
#include <RobotAPI/Test.h>

// ArmarX
#include <ArmarXCore/libraries/cppgen/CppMethod.h>
#include <ArmarXCore/libraries/cppgen/CppClass.h>
#include <RobotAPI/libraries/aron/core/Exception.h>

// Aron
#include <RobotAPI/libraries/aron/core/Debug.h>
#include <RobotAPI/libraries/aron/core/Randomizer.h>

// Generated File
#include <RobotAPI/libraries/aron/core/test/aron/NaturalIKTest.aron.generated.h>

using namespace armarx;
using namespace aron;

BOOST_AUTO_TEST_CASE(AronNavigateTest)
{
    std::cout << "Running navigate test" << std::endl;
    NaturalIKResult k;
    NaturalIKResult k2;

    // test Path
    datanavigator::DictNavigatorPtr aron = k.toAron();
    Path path = aron->getPath(); // should be empty since aron is top level object
    Path memberReached(path, "reached");
    Path memberJointValues(path, "jointValues");
    Path indexJointValues0(memberJointValues, "0");
    Path indexJointValues1(memberJointValues, "1");

    BOOST_CHECK_EQUAL(path.toString(), "\\");
    BOOST_CHECK_EQUAL(path.size(), 0);

    BOOST_CHECK_EQUAL(memberReached.toString(), "\\->reached");
    BOOST_CHECK_EQUAL(memberReached.size(), 1);

    BOOST_CHECK_EQUAL(memberJointValues.toString(), "\\->jointValues");
    BOOST_CHECK_EQUAL(memberJointValues.size(), 1);

    BOOST_CHECK_EQUAL(indexJointValues0.toString(), "\\->jointValues->0");
    BOOST_CHECK_EQUAL(indexJointValues0.size(), 2);

    BOOST_CHECK_EQUAL(indexJointValues1.toString(), "\\->jointValues->1");
    BOOST_CHECK_EQUAL(indexJointValues1.size(), 2);

    datanavigator::BoolNavigatorPtr reached = datanavigator::BoolNavigator::DynamicCastAndCheck(aron->navigateAbsolute(memberReached));
    datanavigator::ListNavigatorPtr jointValues = datanavigator::ListNavigator::DynamicCastAndCheck(aron->navigateAbsolute(memberJointValues));

    if (jointValues->childrenSize() > 0)
    {
        datanavigator::FloatNavigatorPtr el = datanavigator::FloatNavigator::DynamicCastAndCheck(aron->navigateAbsolute(indexJointValues0));
    }
    if (jointValues->childrenSize() > 1)
    {
        datanavigator::FloatNavigatorPtr el = datanavigator::FloatNavigator::DynamicCastAndCheck(aron->navigateAbsolute(indexJointValues1));
    }

    Path diff = indexJointValues1.getWithoutPrefix(indexJointValues0);
    BOOST_CHECK_EQUAL(diff.toString(), "\\->1");
    BOOST_CHECK_EQUAL(diff.size(), 1);
}
